﻿# set up cluster

$product = 'substationSBG'

# we can run this on any system in the network that has permissions, hence asking the user
$node1 = read-host 'Enter local node name'
$node2 = read-host 'Enter other node hostname (system must be online and ready)'

$cluster = Read-Host "Do you want to set up clustering? 'y' for yes, any other to skip"

if ($cluster.tolower().startswith("y")) {

	write-host 'Testing cluster'
    # while not required, it's a very good idea
	Test-Cluster $node1, $node2
	$cluster = read-host 'Cluster name'
	$clusterIP = read-host 'Cluster IP'
	New-Cluster -Name $cluster -Node ($node1, $node2) -NoStorage -StaticAddress "$clusterIP"
}

$account = Read-Host "Do you want to set up the managed service account? 'y' for yes, any other to skip"

if ($account.ToLower().StartsWith("y")) {
    
    #create account
    $dnsHostName = Read-Host "Host name of domain controller"
    $msaAccount = Read-Host "Name for new account (do not include $ or domain)"
    $domainName = Read-Host "Domain name"
    New-ADServiceAccount -DNSHostName $dnsHostName $msaAccount

    $fullmsaAccount = $domainName + "\" + $msaAccount + "$"


    #register account
    Set-ADServiceAccount $msaAccount -PrincipalsAllowedToRetrieveManagedPassword ($node1 + "$")
    Set-ADServiceAccount $msaAccount -PrincipalsAllowedToRetrieveManagedPassword ($node2 + "$")

    #change service to run with account
    $service = gwmi win32_service -computer $node1 -filter "name='$product'"
    $service.change($null,$null,$null,$null,$null,$null,$fullmsaAccount, $null)
    $service = gwmi win32_service -computer $node2 -filter "name='$product'"
    $service.change($null,$null,$null,$null,$null,$null,$fullmsaAccount, $null)

    #update group membership

    #FIXME: automatic way doesn't work on some systems, user will have to run command manually
    #Invoke-Command -ComputerName ($node1, $node2) {net localgroup "substationSBG Admins" /add "$fullmsaAccount"}

    Write-Host "Please run the following command manually on both systems:"
    Write-Host 'net localgroup "substationSBG Admins" /add "'$fullmsaAccount'"'
    pause
}

# copy files

$copyFiles = Read-Host "Do you want to copy the $product configuration files to the other node? 'y' for yes, any other to skip"

if ($copyFiles.ToLower().StartsWith("y"))
{

    Copy-Item -Path "\\$node1\c$\Program Files\$product\*.config" -Destination "\\$node2\c$\Program Files\$product\"
    Copy-Item -Path "\\$node1\c$\Program Files\$product\$product.cer" -Destination "\\$node2\c$\Program Files\$product\"
    Copy-Item -Path "\\$node1\c$\Program Files\$product\Archive" -Destination "\\$node2\c$\Program Files\$product\" -Recurse
    Copy-Item -Path "\\$node1\c$\Program Files\$product\ConfigurationCache" -Destination "\\$node2\c$\Program Files\$product\" -Recurse
    Copy-Item -Path "\\$node1\c$\Program Files\$product\Statistics" -Destination "\\$node2\c$\Program Files\$product\" -Recurse
}

# set up DFS

$dfs = Read-Host "Do you want to set up replication? 'y' for yes, any other to skip"

if ($dfs.tolower().startswith("y"))
{
	write-host 'Creating DFS group'
    $repGroupName = Read-Host 'Replication group name'
	$repGroup = New-DfsReplicationGroup -GroupName $repGroupName #automatically defaults to full mesh
    Add-DfsrMember -GroupName "$repGroupName" -ComputerName $node1, $node2
    Add-DfsrConnection -GroupName $repGroupName -SourceComputerName $node1 -DestinationComputerName $node2 # slightly confusing, but it's actually bi-directional unless we say otherwise
    New-DfsReplicatedFolder -GroupName "$repGroupName" -FolderName ("Archive","Statistics","ConfigurationCache") # add the groups for folders
    Set-DfsrMembership -GroupName "$repGroupName" -FolderName "Archive" -ComputerName $node1, $node2 -ContentPath "C:\Program Files\$product\Archive"
    Set-DfsrMembership -GroupName "$repGroupName" -FolderName "Statistics" -ComputerName $node1, $node2 -ContentPath "C:\Program Files\$product\Statistics"
    Set-DfsrMembership -GroupName "$repGroupName" -FolderName "ConfigurationCache" -ComputerName $node1, $node2 -ContentPath "C:\Program Files\$product\ConfigurationCache"
    Set-DfsrMembership -GroupName "$repGroupName" -FolderName "Archive" -ComputerName $node1 -ContentPath "C:\Program Files\$product\Archive" -PrimaryMember $true
    Set-DfsrMembership -GroupName "$repGroupName" -FolderName "Statistics" -ComputerName $node1 -ContentPath "C:\Program Files\$product\Statistics"  -PrimaryMember $true
    Set-DfsrMembership -GroupName "$repGroupName" -FolderName "ConfigurationCache" -ComputerName $node1 -ContentPath "C:\Program Files\$product\ConfigurationCache"  -PrimaryMember $true
}

# set up failover service

$service = Read-Host "Do you want to set up the failover service? $product must be installed at this point. 'y' for yes, any other to skip"

if ($service.tolower().startswith("y"))
{
    # stop the service, if it's running locally
    net stop $product
    # only user input we need is the IP address and hostname
    $serviceHostname = read-host 'Cluster service hostname'
	$serviceIP = read-host 'Cluster IP'
    Add-ClusterGenericServiceRole -ServiceName "$product" -StaticAddress "$serviceIP" -Name "$serviceHostname"
    # defaults to no disk witness, if there is no cluster storage
}